//-----------------------------------------------------------------------------
// Fnnn_TargetBL_DevSpecific.c
//-----------------------------------------------------------------------------
// Copyright (C) 2010 Silicon Laboratories, Inc.
// http://www.silabs.com
//
// Program Description:
//
// [*** TEMPLATE INFORMATION ***]
// Filename: Fnnn_TargetBL_DevSpecific.c
// nnn => Fill-in with the MCU family part number.
// Examples: F50x_TargetBL_DevSpecific.c, F34x_TargetBL_DevSpecific.c
//
//
// Target:         C8051Fxxx (Any Silicon Labs Flash MCU)
// Tool chain:     Generic
// Command Line:   None
//
//
// Release 1.0 / 01SEP2010 (PKC)
//    -Initial Revision
//
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include <si_toolchain.h>
#include "Fxxx_SFR_Defs_Wrapper.h"     // This header file will include the
                                       // real MCU register definition file

#include "Fxxx_Target_Config.h"
#include "Fxxx_Target_Interface.h"

#include "Fxxx_TargetBL_Config.h"
#include "Fxxx_TargetBL_Interface.h"

//-----------------------------------------------------------------------------
// Global CONSTANTS
//-----------------------------------------------------------------------------

// ------------------------------------------
// *** Device-Specific *** 
// Change this as necessary based on MCU
// ------------------------------------------
#if BOOTLOADER_PIN_OVERRIDE == 1
SI_SBIT(BL_Override_Pin, SFR_P1, 4);
#endif

// Bit masks for the RSTSRC SFR
#define PORSF  0x02
#define FERROR 0x40

#define LIN_TX_PUSH_PULL 0 // 0=Open-drain; 1=Push-pull output

//-----------------------------------------------------------------------------
// Global Variables
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Function Prototypes (Local)
//-----------------------------------------------------------------------------
void OSCILLATOR_Init (void);
void PORT_Init (void);

//=============================================================================
// Function Definitions
//=============================================================================

//-----------------------------------------------------------------------------
// Disable_Watchdog
//-----------------------------------------------------------------------------
//
// Return Value:  None
// Parameters:    None
//
//
//
//-----------------------------------------------------------------------------
void Disable_Watchdog(void)
{
   // *** Device-Specific *** Change this as necessary based on MCU.

   uint8_t SFRPAGE_save = SFRPAGE;
   SFRPAGE = ACTIVE_PAGE;

   PCA0MD &= ~0x40;

   SFRPAGE = SFRPAGE_save;
}

//-----------------------------------------------------------------------------
// Init_Device
//-----------------------------------------------------------------------------
//
// Return Value:  None
// Parameters:    None
//
//
//
//-----------------------------------------------------------------------------
void Init_Device(void)
{
   OSCILLATOR_Init ();
   PORT_Init ();
   Comm_Init ();

   // *** Add calls to any other init functions here as necessary ***
}

#if BOOTLOADER_PIN_OVERRIDE == 1
//-----------------------------------------------------------------------------
// Check_Override_Pin_State
//-----------------------------------------------------------------------------
//
// Return Value:  None
// Parameters:    None
//
//
//
//-----------------------------------------------------------------------------
uint8_t Check_Override_Pin_State (void)
{
   // *** Device-Specific *** 
   // Change this as necessary based on MCU / external hardware.
   
   // Note: P0 through P3 are on all SFR pages, so no page switch code necessary
   // as long as the pin being checked is on one of those ports.

   if (BL_Override_Pin == 0)
   {
      return (MODE_BL_PIN_REQ);
   }
   else
   {
      return (0);
   }
}
#endif //#if BOOTLOADER_PIN_OVERRIDE == 1

//-----------------------------------------------------------------------------
// Check_Reset_Source
//-----------------------------------------------------------------------------
//
// Return Value:  None
// Parameters:    None
//
//
//
//-----------------------------------------------------------------------------
uint8_t Check_Reset_Source (void)
{
   // *** Device-Specific *** 
   // Change this as necessary based on MCU.

   uint8_t retval = 0;
   uint8_t SFRPAGE_save = SFRPAGE;
   SFRPAGE = ACTIVE_PAGE;

   if ((RSTSRC & PORSF) == 0)
   {
      if ((RSTSRC & FERROR) != 0)
      {
         // If the Power-on/Brown-out flag is NOT set,
         // AND if the Flash Error Reset flag is set,
         // this indicates a bootloader request from application firmware
         retval = MODE_BL_APP_REQ;
      }
   }

   SFRPAGE = SFRPAGE_save;
   return (retval);
}


//-----------------------------------------------------------------------------
// Reset_MCU
//-----------------------------------------------------------------------------
//
// Return Value:  None
// Parameters:    None
//
//
//
//-----------------------------------------------------------------------------
void Reset_MCU_Now (void)
{
   // *** Device-Specific *** 
   // Change this as necessary based on MCU.

   SFRPAGE = ACTIVE_PAGE; // No need to save/restore SFR page because of next:
   RSTSRC = 0x12;    // Initiate software reset with vdd monitor enabled   
}

//-----------------------------------------------------------------------------
// OSCILLATOR_Init
//-----------------------------------------------------------------------------
//
// Return Value : None
// Parameters   : None
//
// This routine initializes the system clock to use the internal oscillator
// at its maximum frequency (24 MHz); Also enables the VDD monitor set to its
// high threshold and selected as a reset source.
//
//-----------------------------------------------------------------------------
void OSCILLATOR_Init (void)
{
   // *** Device-Specific *** 
   // TODO: Add this based on MCU.

   uint8_t SFRPAGE_save = SFRPAGE;
   SFRPAGE = CONFIG_PAGE;

   OSCICN |= 0xC7;                     // Configure internal oscillator for
                                       // its maximum frequency

   SFRPAGE = ACTIVE_PAGE;

   VDM0CN = 0xA0;                      // Enable VDD monitor and high threshold

   RSTSRC = 0x02;                      // Enable VDD monitor as a reset source

   SFRPAGE = SFRPAGE_save;
}

//-----------------------------------------------------------------------------
// PORT_Init
//-----------------------------------------------------------------------------
//
// Return Value : None
// Parameters   : None
//
// Configure the Crossbar and GPIO ports.
//
// P1.0   digital  open-drain    LIN0 TX (see condition below)
// P1.1   digital  open-drain    LIN0 RX
//
//-----------------------------------------------------------------------------
void PORT_Init (void)
{
   // *** Device-Specific *** 
   // TODO: Add this based on MCU.

   uint8_t SFRPAGE_save = SFRPAGE;
   SFRPAGE = CONFIG_PAGE;

   P0SKIP   = 0xFF;                    // Skip P0.0 through P0.7

   #if LIN_TX_PUSH_PULL == 1
      P1MDOUT |= 0x01;                 // P1.0 (LIN0 TX) is push-pull
   #else
                                       // P1.0 (LIN0 TX) is open-drain
   #endif
   
   XBR2     = 0x01;                    // Enable LIN0
   XBR2    |= 0x40;                    // Enable crossbar and weak pull-ups

   SFRPAGE = SFRPAGE_save;
}

//-----------------------------------------------------------------------------
// End Of File
//-----------------------------------------------------------------------------
