//-----------------------------------------------------------------------------
// Fxxx_TargetBL_Commands.c
//-----------------------------------------------------------------------------
// Copyright (C) 2010 Silicon Laboratories, Inc.
// http://www.silabs.com
//
// Program Description:
//
// 
//
// How To Test:
//
// 
//
//
// Target:         C8051Fxxx (Any Silicon Labs Flash MCU)
// Tool chain:     Generic
// Command Line:   None
//
//
// Release 1.0 / 01SEP2010 (PKC)
//    -Initial Revision
//
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include <si_toolchain.h>
#include "Fxxx_SFR_Defs_Wrapper.h"     // This header file will include the
                                       // real MCU register definition file

#include "Fxxx_Target_Config.h"
#include "Fxxx_Target_Interface.h"

#include "Fxxx_TargetBL_Config.h"
#include "Fxxx_TargetBL_Interface.h"

//-----------------------------------------------------------------------------
// Global CONSTANTS
//-----------------------------------------------------------------------------



//-----------------------------------------------------------------------------
// Global Variables
//-----------------------------------------------------------------------------
//SI_SEGMENT_VARIABLE(Rx_Buf[RX_BUF_LENGTH], uint8_t, SEG_XDATA);
//SI_SEGMENT_VARIABLE(Tx_Buf[TX_BUF_LENGTH], uint8_t, SEG_XDATA);

uint8_t Rx_Buf[RX_BUF_LENGTH];
uint8_t Tx_Buf[RX_BUF_LENGTH];

uint8_t Flash_Key0 = 0;
uint8_t Flash_Key1 = 0;

#if (TGT_CRC_TYPE & 0x80) == 0 // 16-bit CRC
uint16_t Running_CRC;
#else // 32-bit CRC
uint32_t Runnning_CRC;
#endif

bit Expect_WR_Command_Data_Bytes = 0;

//-----------------------------------------------------------------------------
// Function Prototypes (Local)
//-----------------------------------------------------------------------------


//=============================================================================
// Function Definitions
//=============================================================================

//-----------------------------------------------------------------------------
// Check_App_FW_Validity - Check Signature
//-----------------------------------------------------------------------------
//
// Return Value:  uint8_t - returns value indicating if the App FW is valid.
// Parameters:    None
//
//
//
//-----------------------------------------------------------------------------
uint8_t Check_App_FW_Validity (void)
{
   uint8_t retval = 0;

   #if CODE_BANKING == 0
   SI_VARIABLE_SEGMENT_POINTER(codeptr, uint8_t, SI_SEG_CODE);

   codeptr = (uint8_t SI_SEG_CODE *)(APP_FW_SIG0_ADDR);

   // The Signature (in Flash) should be valid to allow application FW execution.
   // This is written at the end of the bootloading process by the bootloader.
   if (*codeptr-- == SIG_BYTE0)
   {
     if (*codeptr-- == SIG_BYTE1)
     {
         // All signature bytes match.
         retval = MODE_APP_FW_VALID;
      }
   }
   #else
   if (FLASH_ByteRead(APP_FW_SIG0_ADDR) == SIG_BYTE0)
   {
     if (FLASH_ByteRead(APP_FW_SIG0_ADDR-1) == SIG_BYTE1)
     {
         // All signature bytes match.
         retval = MODE_APP_FW_VALID;
      }
   }   
   #endif // #if CODE_BANKING == 0

   return (retval);
}

//-----------------------------------------------------------------------------
// TGT_Enter_BL_Mode
//-----------------------------------------------------------------------------
//
// Return Value:  None
// Parameters:    None
//
//
//
//-----------------------------------------------------------------------------
void TGT_Enter_BL_Mode (void)
{
   // Command Format:
   // [0] Command
   // [1] [dev_serial0]
   // [2] [dev_serial1]
   // ...
   // [7] [dev_serial6]

   // Response:
   // [0] Response code (ACK only; Do not send NACK/ERROR)

   uint8_t bl_ib_len;
   uint8_t app_ib_len = 0;
   #if DEV_SERIAL_NUMBYTES != 0
   bit serial_check_flag = 1;
   uint8_t i;
   #endif

   #if (CODE_BANKING == 0)
   SI_VARIABLE_SEGMENT_POINTER(codeptr, uint8_t, SI_SEG_CODE);

   codeptr = (uint8_t SI_SEG_CODE *)(BL_FW_INFOBLOCK_ADDR-BLIBO_TGT_DEVICE_SERIAL0);
   #endif
   
   #if (DEV_SERIAL_NUMBYTES != 0)
   for (i = 0; i < DEV_SERIAL_NUMBYTES; i++)
   {
      #if CODE_BANKING == 0
      if (Rx_Buf[2+i] != *codeptr--)
      #else
      if (Rx_Buf[2+i] != Flash_ByteRead(BL_FW_INFOBLOCK_ADDR-BLIBO_TGT_DEVICE_SERIAL0-i))
      #endif
      {
         serial_check_flag = 0;
         break;
      }
   }
   #endif

   #if (DEV_SERIAL_NUMBYTES != 0)
   if (serial_check_flag == 1)
   #endif
   {
      BL_Enabled_Flag = BL_ENABLED;   

      if (Rx_Buf[1] == TGT_ENTER_BL_MODE_REQ_RSP_CODE) // Send a response only if byte1 is 0x52
      {
         #if (CODE_BANKING == 0)
         codeptr = (uint8_t SI_SEG_CODE *)(BL_FW_INFOBLOCK_ADDR-BLIBO_TGT_BL_FW_INFOBLOCK_LENGTH);
         bl_ib_len = *codeptr;
         if ((Device_Mode & MODE_APP_FW_VALID) != 0)
         {
            codeptr = (uint8_t SI_SEG_CODE *)(APP_FW_INFOBLOCK_ADDR-APPIBO_TGT_APP_FW_INFOBLOCK_LENGTH);
            app_ib_len = *codeptr;
         }
         #else
         bl_ib_len = Flash_ByteRead(BL_FW_INFOBLOCK_ADDR-BLIBO_TGT_BL_FW_INFOBLOCK_LENGTH);
         if ((Device_Mode & MODE_APP_FW_VALID) != 0)
         {
            app_ib_len = Flash_ByteRead(APP_FW_INFOBLOCK_ADDR-APPIBO_TGT_APP_FW_INFOBLOCK_LENGTH);
         }
         #endif

         Tx_Buf[0] = TGT_RSP_BL_MODE;
         Tx_Buf[1] = bl_ib_len;
         Tx_Buf[2] = app_ib_len;
         Comm_Send_Tx_Bytes (3);
      }
   }
}

//-----------------------------------------------------------------------------
// TGT_Get_Info
//-----------------------------------------------------------------------------
//
// Return Value:  None
// Parameters:    None
//
//
//
//-----------------------------------------------------------------------------
void TGT_Get_Info (void)
{
   // Command Format:
   // [0] Command
   // [1..7] xx

   // Response:
   // [0] Response code (ACK/ERROR etc)
   // [1] BL InfoBlock byte0 (length)
   // [2] BL InfoBlock byte1
   // [.] ...
   // [N] BL InfoBlock byte(N-1)
   // [N+1] App InfoBlock byte0 (length)
   // [N+2] App InfoBlock byte1
   // [...] ...
   // [N+M] App InfoBlock byte(M-1)

   uint8_t numbytes;
   uint8_t i;
   uint8_t index;
   
   #if CODE_BANKING == 0   
   SI_VARIABLE_SEGMENT_POINTER(codeptr, uint8_t, SI_SEG_CODE);
      
   codeptr = (uint8_t SI_SEG_CODE *)(BL_FW_INFOBLOCK_ADDR);
   numbytes = *codeptr;
   #else   
   numbytes = FLASH_ByteRead (BL_FW_INFOBLOCK_ADDR);
   #endif   

   Tx_Buf[0] = TGT_RSP_OK;
    
   index = 1;
   
   for (i = 0; i < numbytes; i++)
   {
      #if CODE_BANKING == 0
      Tx_Buf[index++] = *codeptr;
      codeptr--;
      #else
      Tx_Buf[index++] = FLASH_ByteRead (BL_FW_INFOBLOCK_ADDR-i);
      #endif
   }

   if ((Device_Mode & MODE_APP_FW_VALID) == 0)
   {
      // App FW invalid; Set App FW InfoBlock length in response to 0
      Tx_Buf[index++] = 0;
   }
   else
   {
      #if CODE_BANKING == 0     
      codeptr = (uint8_t SI_SEG_CODE *)(APP_FW_INFOBLOCK_ADDR);
      numbytes = *codeptr;
      #else   
      numbytes = FLASH_ByteRead (APP_FW_INFOBLOCK_ADDR);
      #endif   

      if (numbytes >= (TX_BUF_LENGTH-TGT_BL_FW_INFOBLOCK_LENGTH-1))
      {
         // App FW InfoBlock length invalid; Set App FW InfoBlock length in response to 0
         Tx_Buf[index++] = 0;         
      }
      else
      {         
         for (i = 0; i < numbytes; i++)
         {
            #if CODE_BANKING == 0
            Tx_Buf[index++] = *codeptr;
            codeptr--;
            #else
            Tx_Buf[index++] = FLASH_ByteRead (APP_FW_INFOBLOCK_ADDR-i);
            #endif
         }
      }
   }

   // Note: 'index' should now be equal to the total number of bytes to be sent.

   Comm_Send_Tx_Bytes (index); 
}

//-----------------------------------------------------------------------------
// TGT_Erase_Page
//-----------------------------------------------------------------------------
//
// Return Value:  None
// Parameters:    None
//
//
//
//-----------------------------------------------------------------------------
void TGT_Erase_Page (void)
{
   // Command Format:
   // [0] Command
   // [1] flash key code0
   // [2] flash key code1
   // [3] addr0 (LSB)
   // [4] addr1
   // [5] addr2
   // [6] RESERVED
   // [7] RESERVED

   // Response:
   // [0] Response code

   SI_UU32_t addr;

   addr.u8[b0] = Rx_Buf[3];
   addr.u8[b1] = Rx_Buf[4];
   addr.u8[b2] = Rx_Buf[5];
   addr.u8[b3] = 0;
  
   if ((addr.u32 < APP_FW_START_ADDR) || (addr.u32 > APP_FW_END_ADDR))
   {
      // Address out-of-bounds
      Tx_Buf[0] = TGT_RSP_ADDR_INVALID;   
   }
   else
   {
      Tx_Buf[0] = TGT_RSP_OK;

      Flash_Key0 = Rx_Buf[1];
      Flash_Key1 = Rx_Buf[2];
      Rx_Buf[1] = 0;
      Rx_Buf[2] = 0;

      #if CODE_BANKING == 0
      FLASH_PageErase (addr.u16[LSB]);
      #else
      FLASH_PageErase (addr.u32);
      #endif      

      // Set flash keys to 0
      Flash_Key0 = 0;
      Flash_Key1 = 0;
            
      // If previously valid, re-check if App FW is still valid (==> checks if signature was erased)
      if ((Device_Mode & MODE_APP_FW_VALID) != 0)
      {
         if (Check_App_FW_Validity() == 0)
         {
            Device_Mode &= ~MODE_APP_FW_VALID;
         }
      }
   }
   
   Comm_Send_Tx_Bytes (1);   
}

//-----------------------------------------------------------------------------
// TGT_Write_Flash
//-----------------------------------------------------------------------------
//
// Return Value:  None
// Parameters:    None
//
//
//
//-----------------------------------------------------------------------------
void TGT_Write_Flash (void)
{
   // Command Format:
   // [0] Command
   // [1] flash key code0
   // [2] flash key code1
   // [3] addr0 (LSB)
   // [4] addr1
   // [5] addr2
   // [6] numbytes0 (LSB)
   // [7] numbytes1
   
   // Response:
   // [0] Response code

   // Bytes to write:
   // [0] byte0
   // [1] byte1
   // [.] ...
   // [numbytes-1] byte(numbytes-1)

   SI_UU32_t start_addr;
   SI_UU32_t end_addr;
   SI_UU16_t numbytes;
   bit proceed_flag = 0;
   uint16_t i;

   start_addr.u8[b0] = Rx_Buf[3];
   start_addr.u8[b1] = Rx_Buf[4];
   start_addr.u8[b2] = Rx_Buf[5];
   start_addr.u8[b3] = 0;

   numbytes.u8[LSB] = Rx_Buf[6];
   numbytes.u8[MSB] = Rx_Buf[7];

   end_addr.u32 = start_addr.u32 + numbytes.u16 - 1;
  
   // Write to flash only if App FW signature was previously erased        
   if ((Device_Mode & MODE_APP_FW_VALID) != 0)
   {
      // Error: Signature has not yet been erased
      Tx_Buf[0] = TGT_RSP_SIG_NOT_ERASED;
   }
   else if ((start_addr.u32 < APP_FW_START_ADDR) || (start_addr.u32 > APP_FW_INFOBLOCK_ADDR)
            || (end_addr.u32 < APP_FW_START_ADDR) || (end_addr.u32 > APP_FW_INFOBLOCK_ADDR))
   {
      // Address out-of-bounds
      Tx_Buf[0] = TGT_RSP_ADDR_INVALID;   
   }
   else if (numbytes.u16 > RX_BUF_LENGTH)
   {
      // Too many bytes!
      Tx_Buf[0] = TGT_RSP_PARAMETER_INVALID;        
   }
   else
   {
      // Store relevant information
      Flash_Key0 = Rx_Buf[1];
      Flash_Key1 = Rx_Buf[2];
      Rx_Buf[1] = 0;
      Rx_Buf[2] = 0;
      proceed_flag = 1;
      Tx_Buf[0] = TGT_RSP_OK;
   }

   // Send response
   Comm_Send_Tx_Bytes (1);   

   if (proceed_flag == 1)
   {
      Expect_WR_Command_Data_Bytes = 1;

      // Get the bytes to be written to flash
      Comm_Wait_For_Rx_Bytes (numbytes.u16);

      Expect_WR_Command_Data_Bytes = 0;

      for (i = 0; i < numbytes.u16; i++)
      {
         #if CODE_BANKING == 0
         FLASH_ByteWrite (start_addr.u16[LSB], Rx_Buf[i]);
         start_addr.u16[LSB] += 1;
         #else
         FLASH_ByteWrite (start_addr.u32, Rx_Buf[i]);
         start_addr.u32 += 1;
         #endif
      }      

      // Set flash keys to 0
      Flash_Key0 = 0;
      Flash_Key1 = 0;

      Tx_Buf[0] = TGT_RSP_OK;
      Comm_Send_Tx_Bytes (1);   
   } 
}

//-----------------------------------------------------------------------------
// TGT_Get_Page_CRC
//-----------------------------------------------------------------------------
//
// Return Value:  None
// Parameters:    None
//
//
//
//-----------------------------------------------------------------------------
void TGT_Get_Page_CRC (void)
{
   // Command Format:
   // [0] Command
   // [1] addr0 (LSB)
   // [2] addr1
   // [3] addr2
   // [4] RESERVED
   // [5] xx
   // [6] xx
   // [7] xx

   // Response:
   // [0] Response code
   // [1] crc0 (LSB)
   // [2] crc1
   // [3] [crc2]
   // [4] [crc3]

   SI_UU32_t addr;
   SI_UU32_t crc;
   uint16_t i;
   
   #if CODE_BANKING == 0
   SI_VARIABLE_SEGMENT_POINTER(codeptr, uint8_t, SI_SEG_CODE);
   #endif   

   addr.u8[b0] = Rx_Buf[1];
   addr.u8[b1] = Rx_Buf[2];
   addr.u8[b2] = Rx_Buf[3];
   addr.u8[b3] = 0;
  
   if ( (addr.u32 < APP_FW_START_ADDR) || (addr.u32 > APP_FW_END_ADDR) || 
         ((addr.u32 & TGT_FLASH_PAGE_ADDR_MASK) != 0)
      )
   {
      // Address out-of-bounds
      Tx_Buf[0] = TGT_RSP_ADDR_INVALID;  
      // crc bytes sent are "don't care".
   }
   else
   {
      Running_CRC = 0; // Init Running CRC to 0
      
      #if CODE_BANKING == 0
      codeptr = (uint8_t SI_SEG_CODE *)(addr.u16[LSB]);      
      #endif
      
      for (i = 0; i < TGT_FLASH_PAGE_SIZE; i++)
      {
         #if CODE_BANKING == 0
            Update_CRC (*codeptr);
            codeptr++;
         #else
            Update_CRC (FLASH_ByteRead(addr.u32));
            addr.u32++;
         #endif
      }   

      Tx_Buf[0] = TGT_RSP_OK;

      #if (TGT_CRC_TYPE & 0x80) == 0 // 16-bit CRC
         crc.u16[LSB] = Running_CRC;
         Tx_Buf[1] = crc.u8[b0];
         Tx_Buf[2] = crc.u8[b1];
      #else // 32-bit CRC
         crc.u32 = Running_CRC;
         Tx_Buf[3] = crc.u8[b2];
         Tx_Buf[4] = crc.u8[b3];      
      #endif
   }
   
   #if (TGT_CRC_TYPE & 0x80) == 0 // 16-bit CRC
      Comm_Send_Tx_Bytes (3);   
   #else   
      Comm_Send_Tx_Bytes (5);   
   #endif
}

//-----------------------------------------------------------------------------
// TGT_Write_Signature
//-----------------------------------------------------------------------------
//
// Return Value:  None
// Parameters:    None
//
//
//
//-----------------------------------------------------------------------------
void TGT_Write_Signature (void)
{
   // Command Format:
   // [0] Command
   // [1] flash key code0
   // [2] flash key code1
   // [3] signature byte0
   // [4] signature byte1
   // [5] RESERVED
   // [6] RESERVED
   // [7] RESERVED

   // Response:
   // [0] Response code

   if ((Device_Mode & MODE_APP_FW_VALID) != 0)
   {
      // Error: Signature has not yet been erased
      Tx_Buf[0] = TGT_RSP_SIG_NOT_ERASED;
   }
   else
   {
      Flash_Key0 = Rx_Buf[1];
      Flash_Key1 = Rx_Buf[2];
      Rx_Buf[1] = 0;
      Rx_Buf[2] = 0;

      FLASH_ByteWrite (APP_FW_SIG0_ADDR, Rx_Buf[3]);
      FLASH_ByteWrite (APP_FW_SIG0_ADDR-1, Rx_Buf[4]);

      // Set flash keys to 0
      Flash_Key0 = 0;
      Flash_Key1 = 0;

      Tx_Buf[0] = TGT_RSP_OK;
   }
   
   Comm_Send_Tx_Bytes (1);
}

//-----------------------------------------------------------------------------
// TGT_Reset_MCU
//-----------------------------------------------------------------------------
//
// Return Value:  None
// Parameters:    None
//
//
//
//-----------------------------------------------------------------------------
void TGT_Reset_MCU (void)
{
   // Command Format:
   // [0] Command
   // [1..7] xx

   // Response:
   // [0] Response code

   Tx_Buf[0] = TGT_RSP_OK;
   Comm_Send_Tx_Bytes (1);
   
   // Add delay here to allow time for the Comm to send response
   Running_CRC = 0xFFFF; 
   while (Running_CRC-- != 0);

   Reset_MCU_Now (); // *** This function call will reset the MCU ***
}



//-----------------------------------------------------------------------------
// TGT_Unsupported_Cmd
//-----------------------------------------------------------------------------
//
// Return Value:  None
// Parameters:    None
//
//
//
//-----------------------------------------------------------------------------
void TGT_Unsupported_Cmd (void)
{
   // Command Format:
   // [0] Command
   // [1..7] xx

   // Response:
   // [0] Response code

   Tx_Buf[0] = TGT_RSP_UNSUPPORTED_CMD;
   Comm_Send_Tx_Bytes (1);
}

//-----------------------------------------------------------------------------
// End Of File
//-----------------------------------------------------------------------------
