//-----------------------------------------------------------------------------
// Fxxx_TargetBL_Main.c
//-----------------------------------------------------------------------------
// Copyright (C) 2010 Silicon Laboratories, Inc.
// http://www.silabs.com
//
// Program Description:
//
// The main routine for the Target Bootloader Firmware.
//
// 
//
//
// Target:         C8051Fxxx (Any Silicon Labs Flash MCU)
// Tool chain:     Generic
// Command Line:   None
//
//
// Release 1.0 / 01SEP2010 (PKC)
//    -Initial Revision
//
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include <si_toolchain.h>
#include "Fxxx_SFR_Defs_Wrapper.h"     // This header file will include the
                                       // real MCU register definition file

#include "Fxxx_Target_Config.h"
#include "Fxxx_Target_Interface.h"

#include "Fxxx_TargetBL_Config.h"
#include "Fxxx_TargetBL_Interface.h"

//-----------------------------------------------------------------------------
// Global CONSTANTS
//-----------------------------------------------------------------------------



//-----------------------------------------------------------------------------
// Global Variables
//-----------------------------------------------------------------------------
uint8_t Device_Mode = 0;
bit BL_Enabled_Flag = BL_DISABLED;

//-----------------------------------------------------------------------------
// Function Prototypes (Local)
//-----------------------------------------------------------------------------


//=============================================================================
// Main Routine
//=============================================================================
void main (void)
{
   //---------------------------------------
   // Local Variable Definitions
   //---------------------------------------
   uint8_t cmd_code;

   //---------------------------------------
   // Perform basic MCU initialization
   //---------------------------------------

   Disable_Watchdog();

   EA = 0;  // Disable all interrupts
   EA = 0;  // Some MCUs require this instruction repeated (see MCU datasheet)

   //---------------------------------------
   // Check App FW Signature/CRC
   //---------------------------------------

   Device_Mode |= Check_App_FW_Validity();

   //---------------------------------------
   // Check the override pin, if enabled
   //---------------------------------------

   // Check the Bootloader-Override Pin on the device if this feature is enabled
   #if BOOTLOADER_PIN_OVERRIDE == 1
      // If pin is asserted, enter bootload mode and ignore other conditions.
      Device_Mode |= Check_Override_Pin_State();
   #endif

   //---------------------------------------
   // Check the reset source
   //---------------------------------------

   // If the reset was a App BL request, stay in BL mode
   Device_Mode |= Check_Reset_Source();

   //---------------------------------------
   // Launch application firmware (
   //---------------------------------------

   if (((Device_Mode & MODE_APP_FW_VALID) != 0) && 
      ((Device_Mode & (MODE_BL_PIN_REQ | MODE_BL_APP_REQ)) == 0))
   {
      // If app FW is valid and if no FW update request was detected, jump to app FW
      START_APPLICATION ();
   }

   //-------------------------------------------
   // ** BL Mode ** Initialize MCU and Variables
   //-------------------------------------------

   Init_Device();

   //-------------------------------------------
   // Main Loop
   //-------------------------------------------

   while(1)
   {
      // Wait until a command is received
      cmd_code = Comm_Wait_For_Rx_Bytes (TGT_CMD_FIXED_LENGTH);

      // Process command
      if (cmd_code == TGT_CMD_ENTER_BL_MODE)
      {
         // Will enter BL Mode only if device serial is matched 
         // (if that feature is enabled)
         TGT_Enter_BL_Mode ();
         // Will not send any response to any commands until this 
         // command has been issued at least once while in bootload mode.
      }
      else if (BL_Enabled_Flag == BL_ENABLED)
      {
         switch (cmd_code)
         {
            case TGT_CMD_GET_INFO:
               TGT_Get_Info ();
               break;
            case TGT_CMD_ERASE_FLASH_PAGE:
               TGT_Erase_Page ();
               break;
            case TGT_CMD_WRITE_SIGNATURE:
               TGT_Write_Signature ();
               break;
            case TGT_CMD_RESET_MCU:
               TGT_Reset_MCU ();
               break;
            case TGT_CMD_GET_PAGE_CRC:
               TGT_Get_Page_CRC ();
               break;
            case TGT_CMD_WRITE_FLASH_BYTES:
               TGT_Write_Flash ();
               break;
            default:
               TGT_Unsupported_Cmd ();
               break;
         }
      }
   }
}


//-----------------------------------------------------------------------------
// End Of File
//-----------------------------------------------------------------------------
